
/*
	Playlist editor plugin for SoundPlay
	Copyright 2000 Marco Nelissen (marcone@xs4all.nl)

	Permission is hereby granted to use this code for creating other
	SoundPlay-plugins.
*/

#include "Editor.h"
const int left = 50;
const int top = 50;
const int width=500;
const int height=600;

const int RightPaneWidth=100;
Editor::Editor(PlaylistPtr pl)
	: BWindow(BRect(left,top,left+width,top+height),"Playlist",B_TITLED_WINDOW_LOOK,B_NORMAL_WINDOW_FEEL,0)
{
	BView* panelView = new BView(Bounds(),"panelView",B_FOLLOW_ALL_SIDES,B_WILL_DRAW);
	panelView->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	panelView->SetLowColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	AddChild(panelView);
	
	playlist=pl;
	remoteapp = BMessenger("application/x-vnd.marcone-soundplay");
	lv=new PlayListView(BRect(0,0,width-B_V_SCROLL_BAR_WIDTH-RightPaneWidth,height));
	panelView->AddChild(new BScrollView("",lv,B_FOLLOW_ALL_SIDES,B_WILL_DRAW,false,true));
	//float w = width-B_V_SCROLL_BAR_WIDTH;
	BRect r = Bounds();
	int spacing=10;
	int btn_width=20;
	BRect btn = BRect(r.right-RightPaneWidth+spacing,spacing,r.right-spacing,spacing+btn_width);
	sort_name = new BButton(btn,"sort_name","Sort by Name",new BMessage(SORT_NAME),B_FOLLOW_TOP|B_FOLLOW_RIGHT);
	panelView->AddChild(sort_name);

	btn.OffsetBy(0,btn_width+spacing);
	sort_path = new BButton(btn,"sort_path","Sort by Path",new BMessage(SORT_PATH),B_FOLLOW_TOP|B_FOLLOW_RIGHT);
	panelView->AddChild(sort_path);
	
	btn.OffsetBy(0,btn_width+spacing);
	randomize = new BButton(btn,"randomize","Randomize",new BMessage(RANDOMIZE),B_FOLLOW_TOP|B_FOLLOW_RIGHT);
	panelView->AddChild(randomize);
	
	btn.OffsetBy(0,btn_width+spacing);
	remove = new BButton(btn,"remove","Remove",new BMessage(REMOVE),B_FOLLOW_TOP|B_FOLLOW_RIGHT);
	panelView->AddChild(remove);
	
	btn.OffsetBy(0,btn_width+spacing);
	remove_all = new BButton(btn,"remove_all","Remove All",new BMessage(REMOVE_ALL),B_FOLLOW_TOP|B_FOLLOW_RIGHT);
	panelView->AddChild(remove_all);

	btn.OffsetBy(0,btn_width+spacing);
	vol = new VolBar(btn);	
	panelView->AddChild(vol);
	
	btn.OffsetBy(0,btn_width+(spacing*2));
	prog = new ProgBar(btn);	
	panelView->AddChild(prog);


	btn.OffsetBy(0,btn_width+(spacing*3));
	play = new BButton(btn,"play","Play",new BMessage(PLAY),B_FOLLOW_TOP|B_FOLLOW_RIGHT);
	panelView->AddChild(play);
	
	btn.OffsetBy(0,btn_width+spacing);
	paws = new BButton(btn,"paws","Pause",new BMessage(PAUSE),B_FOLLOW_TOP|B_FOLLOW_RIGHT);
	panelView->AddChild(paws);
	
	btn.OffsetBy(0,btn_width+spacing);
	plyp = new BButton(btn,"plyp","Play Previous",new BMessage(PLAYPREV),B_FOLLOW_TOP|B_FOLLOW_RIGHT);
	panelView->AddChild(plyp);
	
	btn.OffsetBy(0,btn_width+spacing);
	plyn = new BButton(btn,"plyn","Play Next",new BMessage(PLAYNEXT),B_FOLLOW_TOP|B_FOLLOW_RIGHT);
	panelView->AddChild(plyn);
	
	btn.OffsetBy(0,btn_width+spacing);
	scrolltoselection = new BButton(btn,"scrolltoselection","View Current",new BMessage(SCROLLTOSELECTION),B_FOLLOW_TOP|B_FOLLOW_RIGHT);
	panelView->AddChild(scrolltoselection);
	
	
	playlist->Lock();
	playlist->AddListener(this);
	Populate();
	vol->Update(playlist->Volume());
	file_info fi;
	playlist->GetInfo(playlist->CurrentID(),&fi);
	prog->SetMaxValue(fi.framecount / fi.samplerate);

	playlist->Unlock();
	
	BMessenger msgr(this);
	BMessage check_vol(RUNR);
	//sends lots of BMessages (STARTME) (1 every UPDATE_TIME µs) infinitely (-1)
	volRunner = new BMessageRunner(msgr,&check_vol,UPDATE_TIME,-1);
	
	Show();
}
void Editor::FrameResized(float width,float height)
{
	int current = lv->CurrentSelection();
	if(current >= 0)
	{
		lv->InvalidateItem(current);
	}
}
void Editor::FrameResized(BPoint point)
{
	int current = lv->CurrentSelection();
	if(current >= 0)
	{
		lv->InvalidateItem(current);
	}
}
/*
Editor::~Editor()
{
}
*/

void Editor::MessageReceived(BMessage *msg)
{
	
	switch(msg->what)
	{
		case SCROLLTOSELECTION:
		{
			lv->ScrollToSelection();
		}break;
		
		case PLAY:
		{
			if(playlist->Lock())
			{
				playlist->Play();
				playlist->Unlock();
			}		
		}break;
		case PAUSE:
		{
			if(playlist->Lock())
			{
				playlist->Pause();
				playlist->Unlock();
			}		
		}break;
		case PLAYPREV:
		{
			if(playlist->Lock())
			{
				playlist->PlayPrevious();
				playlist->Unlock();
			}		
		}break;
		case PLAYNEXT:
		{
			if(playlist->Lock())
			{
				playlist->PlayNext();
				playlist->Unlock();
			}		
		}break;
					
		case REQUEST_PROG_CHANGE:
		{
			float pos;
			if(msg->FindFloat("prog",&pos) == B_OK)
			{
				if(playlist->Lock() && pos <= prog->MaxValue() && pos >= 0)
				{
					playlist->SetPosition(pos);
					playlist->Unlock();
				}
			}
		
		}break;
		case REQUEST_VOL_CHANGE:
		{
			float vol_lvl;
			if(msg->FindFloat("vol",&vol_lvl) == B_OK)
			{
				if(playlist->Lock())
				{
					playlist->SetVolume(vol_lvl);
					playlist->Unlock();
				}
			}
		}break;
		case RUNR:
		{
			
			if(playlist->Lock())
			{
				float cur = vol->CurrentValue();
				//if(cur <= vol->MaxValue() && cur >= 0)
				//{
					float pv = playlist->Volume();
					
					if(pv > cur)
						vol->Update(pv - cur);
					else if(pv < cur)
						vol->Update(-(cur - pv));
				//}
				cur = prog->CurrentValue();
				if(cur <= prog->MaxValue() && cur >= 0)
				{
					double pp = playlist->Position();
					if(pp > cur)
						prog->Update(pp - cur);
					else if(pp < cur)
						prog->Update(-(cur - pp));
				}
					
				//cout << playlist->Position() << endl;
				playlist->Unlock();
			}
		}break;
		case SORT_NAME: 
		{
			if(playlist->Lock())
			{
				playlist->Sort();
				playlist->Unlock();
			}		
		} break;
		case SORT_PATH:
		{
			if(playlist->Lock())
			{
				playlist->SortByPath();
				playlist->Unlock();
			}
		
		
		} break;
		case RANDOMIZE: 
		{
			if(playlist->Lock())
			{
				playlist->Shuffle();			
				playlist->Unlock();
			}
		
		
		} break;
		case REMOVE: 
		{
			if(playlist->Lock())
			{
				int currentID = playlist->CurrentID();
				//int currentIDX = playlist->CurrentIndex();
				if(playlist->CountItems() > 0)
				{
					playlist->Remove(currentID);
					/*if(currentIDX >= 0)
					{
						currentIDX + 1*/
				}
				playlist->Unlock();
			}
		
		
		} break;
		case REMOVE_ALL: 
		{
			if(playlist->Lock())
			{
				playlist->MakeEmpty();
				playlist->Unlock();
			}
		
		
		} break;     		
		
		
		case PLAYLIST_CURRENT:		
		if(playlist->Lock())
		{	
			
			int32 id;
			int32 plid;
			if(msg->FindInt32("Playlist",&plid) == B_OK)
			{
				PlaylistPtr pl(plid);
				if(pl->IsValid())
				{
					int32 current = pl->CurrentIndex();
					if(current >= 0)
					lv->Select(current);
				}
			}
			else if(msg->FindInt32("ID",&id) == B_OK) 
			{
				
				if(id != playlist->CurrentID())
				playlist->PlayFile(id);
				
			}
			file_info fi;
			playlist->GetInfo(id,&fi);
			prog->Reset();
			prog->SetMaxValue(fi.framecount / fi.samplerate);

			playlist->Unlock();
		}
		break;
		case PLAYLIST_EMPTY:
		
			MakeEmpty();
			break;

		case PLAYLIST_SORTED:
			Populate();
			
			break;

		case PLAYLIST_ADD:
			if(playlist->Lock()){
			for(int i=0;;i++)
			{
				int32 index;
				int32 id;
				if(B_OK==msg->FindInt32("ID",i,&id)
				 &&B_OK==msg->FindInt32("index",i,&index))
				 {
					//file_info fi;
					//playlist->GetInfo(id,&fi);
					
					lv->AddItem(new PlayListItem(id,playlist->NameForItem(id),index % 2 == 0/*,fi.framecount / fi.samplerate*/));
				}
				else
					break;
			}
			
			int32 current = playlist->CurrentIndex();
			if(current >= 0)
				lv->Select(current);
		
			playlist->Unlock();
		}
			break;

		case PLAYLIST_REMOVE:
			playlist->Lock();
			for(int i=0;;i++)
			{
				int32 id;
				if(B_OK==msg->FindInt32("ID",i,&id))
				{
					for(int i=0;;i++)
					{
						PlayListItem *item;
						item=(PlayListItem*)lv->ItemAt(i);
						if(!item)
							break;
						if(item->id==id)
						{
							lv->RemoveItem(item);
							delete item;
						}
					}
				}
				else
					break;
			}
			playlist->Unlock();
			break;

		case PLAYLIST_ITEM_RENAMED:
			playlist->Lock();
			for(int i=0;;i++)
			{
				int32 id;
				if(B_OK==msg->FindInt32("ID",i,&id))
				{
					for(int i=0;;i++)
					{
						PlayListItem *item;
						item=(PlayListItem*)lv->ItemAt(i);
						if(!item)
							break;
						if(item->id==id)
						{
							item->SetLabel(playlist->NameForItem(id));
							lv->InvalidateItem(i);
						}
					}
				}
				else
					break;
			}
			playlist->Unlock();
			break;

		default:
			BWindow::MessageReceived(msg);
			break;
	}
}

void Editor::MakeEmpty()
{
	PlayListItem *item;
	int32 numitems=lv->CountItems();
	while(--numitems>=0)
	{
		item=(PlayListItem*)lv->RemoveItem(numitems);
		delete item;
	}
}


void Editor::Populate()
{
	MakeEmpty();

	playlist->Lock();
	for(int i=0;;i++)
	{
		int32 id=playlist->IDForItemAt(i);
		if(id<0)
			break;

		//file_info fi;
		//playlist->GetInfo(id,&fi);
		
		lv->AddItem(new PlayListItem(id,playlist->NameForItem(id),i % 2 == 0/*, fi.framecount / fi.samplerate*/));
		if(i == playlist->CurrentIndex())
		{
			lv->Select(i);			
		}
	}
	playlist->Unlock();
}
/*BMessenger Editor::RemoteTrack(int32 index)
{
	BMessage question(B_GET_PROPERTY);
	question.AddSpecifier("Messenger");  // a track is a BHandler, and therefore supports the handler scripting suite
	question.AddSpecifier("track",index);
	BMessage answer;
	BMessenger remotetrack;

	if(B_OK == remoteapp.SendMessage(&question, &answer,2E6,2E6))
	{
		int32 error;
		if(B_OK == answer.FindInt32("error",&error) && error == B_OK && answer.HasMessenger("result"))
			answer.FindMessenger("result",&remotetrack);
	}
	return remotetrack;
}
*/
PlayListItem::PlayListItem(int32 itemid, const char *name,bool IsEven/*,double l*/)
	: BListItem()
{
	label = name;
	id=itemid;
	//length = l;
	
	//ItemColor = ;//tint_color(/*ui_color(B_WINDOW_TAB_COLOR)*/,B_LIGHTEN_2_TINT);
	/*if(IsEven)
		ItemColor = tint_color(ui_color(B_WINDOW_TAB_COLOR),B_LIGHTEN_1_TINT);
	else
		ItemColor = tint_color(ui_color(B_WINDOW_TAB_COLOR),B_LIGHTEN_2_TINT);
	*/
}
BString PlayListItem::Label()
{
	return label;
}
void PlayListItem::DrawItem(BView* owner,BRect frame,bool complete=false)
{
	rgb_color color;
	color = owner->ViewColor();
	if(IsSelected() || complete)
	{
		if(IsSelected())
		{
		BFont font(be_bold_font);
		font.SetSize(12);
		owner->SetFont(&font);
			color = ui_color(B_WINDOW_TAB_COLOR);//owner->ViewColor();//highlight_color;
		}
		else
		{ 
			BFont font(be_plain_font);
			font.SetSize(12);
			owner->SetFont(&font);
			color = owner->ViewColor();
		}
		owner->SetHighColor(color);
		owner->SetLowColor(color);
		//owner->SetLowColor(255,255,255);
		owner->FillRect(frame);
	}
	else
	{
		BFont font(be_plain_font);
		font.SetSize(12);
		owner->SetFont(&font);	
		owner->SetHighColor(color);
		owner->SetLowColor(color);
		//owner->SetLowColor(255,255,255);
		owner->FillRect(frame);
	}
	owner->MovePenTo(frame.left+4,frame.bottom-2);
	
	if(IsEnabled())
	{
		//set highcolor for font
		owner->SetHighColor(0,0,0);
	}
	else
	{
		//set highcolor for font
		owner->SetHighColor(0,0,0);
		//owner->SetLowColor(255,255,255);
		
	}
	
	owner->SetLowColor(color);
	owner->DrawString(label.String());
/*	owner->MovePenTo(frame.right-20,frame.bottom-2);
	
	duration = "";
	int seconds = int(length);
	int minutes = seconds / 60;
	seconds = seconds % 60;
	
	if(minutes < 10 && seconds >= 10)
		duration << "0" << (int)minutes << ":" << (int)seconds;
	else if(minutes < 10 && seconds < 10)
		duration << "0" << (int)minutes << ":0" << (int)seconds;
	else if(minutes >= 10 && seconds < 10)
		duration << (int)minutes << ":0" << (int)seconds;
	else
		duration << (int)minutes << ":" << (int)seconds;
	owner->DrawString(duration.String());
*/
/*	rgb_color color;
	const int fsize=12;
	
	color = owner->ViewColor();
	if(IsSelected() || complete)
	{
		if(IsSelected())
		{
			if(GRADIENT_LIST_ITEM)
			{
				BRect r = frame;
				float ItemHeight = r.bottom;
				rgb_color low;
				for(int i=0;i<=r.Height();i++)
				{
					owner->SetHighColor(255,i*int(255/r.Height()),0);
				
					if(i == (int)r.Height()/2)
						low = owner->HighColor();
					//SetDrawingMode(B_OP_INVERT);
					//SetHighColor(tint_color(HighColor(),B_DARKEN_1_TINT));
					//}	
					owner->StrokeLine(BPoint(r.left+1,ItemHeight-i),BPoint(r.right-1,ItemHeight-i));
					
					if(i == 0)
					{	
						owner->SetHighColor(tint_color(owner->HighColor(),B_DARKEN_2_TINT));
						owner->StrokeLine(BPoint(r.left+1,ItemHeight-i),BPoint(r.right-1,ItemHeight-i));
					}
					else if(i == (int)r.Height())
					{	
						owner->SetHighColor(tint_color(owner->HighColor(),B_LIGHTEN_2_TINT));
						owner->StrokeLine(BPoint(r.left+1,ItemHeight-i),BPoint(r.right-1,ItemHeight-i));
					}
					
					//left highlight
					owner->SetHighColor(tint_color(owner->HighColor(),B_LIGHTEN_2_TINT));
					owner->StrokeLine(BPoint(r.left,ItemHeight-i),BPoint(r.left,ItemHeight-i));
					//right shadow
					owner->SetHighColor(tint_color(owner->HighColor(),B_DARKEN_2_TINT));
					owner->StrokeLine(BPoint(r.right,ItemHeight-i),BPoint(r.right,ItemHeight-i));
					
					GRADIENT_LIST_ITEM = true;
					
					BFont font(be_bold_font);
					font.SetSize(fsize);
					owner->SetFont(&font);
					
					owner->SetHighColor(0,0,0);
					owner->SetLowColor(low);
					owner->MovePenTo(frame.left+4,frame.bottom-2);
					owner->DrawString(label.String());
					
				}
			}
				else
				{
					BFont font(be_bold_font);
					font.SetSize(fsize);
					owner->SetFont(&font);
					
					color = ui_color(B_WINDOW_TAB_COLOR);//owner->ViewColor();//highlight_color;
							
				}		
		
		}
		else
		{ 
			BFont font(be_plain_font);
			font.SetSize(fsize);
			owner->SetFont(&font);
			//color = ItemColor;
			color = owner->ViewColor();
			owner->SetHighColor(color);
			owner->SetLowColor(color);
			//owner->SetLowColor(255,255,255);
			owner->FillRect(frame);

		}
	}
	else
	{
		BFont font(be_plain_font);
		font.SetSize(fsize);
		owner->SetFont(&font);
		
		owner->SetHighColor(color);
		owner->SetLowColor(color);
		//owner->SetLowColor(255,255,255);
		owner->FillRect(frame);

	}
	if(!GRADIENT_LIST_ITEM)
	{
		owner->MovePenTo(frame.left+4,frame.bottom-2);
		
		if(IsEnabled())
		{
			//set highcolor for font
			owner->SetHighColor(0,0,0);
		}
		else
		{
			//set highcolor for font
			owner->SetHighColor(0,0,0);
			//owner->SetLowColor(255,255,255);
			
		}
		owner->SetLowColor(color);
		
		owner->DrawString(label.String());
	}*/
}
void PlayListItem::SetLabel(const char* l)
{
	label = l;
}	
/*void PlayListItem::Play()
{

	//printf("Play: %s %d\n",label.String(),id);
	
}*/
PlayListView::PlayListView(BRect frame) : BListView(frame,"PlayListView",B_SINGLE_SELECTION_LIST,B_FOLLOW_ALL_SIDES,B_WILL_DRAW|B_FRAME_EVENTS|B_NAVIGABLE)
{
	//OkToSend = true;
}
/*void PlayListView::Select(int32 index,bool IsManual,bool extend=false)
{
	OkToSend = IsManual;
	BListView::Select(index,extend);
}*/
void PlayListView::AttachedToWindow()
{
	MakeFocus();
}
void PlayListView::KeyDown(const char* bytes,int32 numBytes)
{
	if(isalnum(bytes[0]))
		HotKeySelect(bytes[0]);
	BListView::KeyDown(bytes,numBytes);
}
void PlayListView::HotKeySelect(char c)
{
	int GoToIndex=-1;
	char Upper = toupper(c);
		//start from front
		for(int i=0;i<CountItems();i++)
		{
			if(toupper(((PlayListItem*)ItemAt(i))->Label().ByteAt(0)) == Upper)
			{	
				GoToIndex = i;
				break;
			}
		}
	if(GoToIndex >= 0)
		Select(GoToIndex);
}
void PlayListView::MouseDown(BPoint point)
{
	//GetMouse(&point,&button);
	BListView::MouseDown(point);
}
void PlayListView::SelectionChanged()
{
	//if(OkToSend)
	//{
	//if(button == B_PRIMARY_MOUSE_BUTTON)
	//{
		int current = CurrentSelection();
		if(current >= 0)
		{	
			BMessage msg(PLAYLIST_CURRENT);
			if(msg.AddInt32("ID",((PlayListItem*)ItemAt(current))->id) == B_OK)			
			Window()->PostMessage(&msg);
			
		}
		//button = B_SECONDARY_MOUSE_BUTTON;
	//}	
	//}
	//else
	//	OkToSend = true;
	ScrollToSelection();
	BListView::SelectionChanged();
}