
/*
	Playlist editor plugin for SoundPlay
	Copyright 2000 Marco Nelissen (marcone@xs4all.nl)

	Permission is hereby granted to use this code for creating other
	SoundPlay-plugins.
*/

#include <Application.h>
#include <Roster.h>
#include <AppFileInfo.h>
#include <PopUpMenu.h>
#include <MenuItem.h>
#include <stdio.h>
#include <string>
#include "pluginproto.h"
#include "Editor.h"

static const char *PLUGINID="PlaylistEditor";
static void *getplugin(void **data,const char *name, const char *, uint32, plugin_info *, SoundPlayBufferedReader *reader);
static void destroyplugin(void*,void *_loader);

// =======================================================================
// standard plugin stuff
plugin_descriptor plugin = {
	PLUGIN_INTERFACE_MAGIC,PLUGIN_INTERFACE_VERSION,
	PLUGINID,1, PLUGIN_IS_INTERFACE,
	"SimplePlaylist",
	"An improvement on the 'Unfinished Playlist' bundled with SoundPlay.\nBy Brent Miszalski.",
	NULL,
	NULL,
	&getplugin,
	&destroyplugin
};

static interface_plugin_ops plugin_ops={
	PLUGIN_FILTER_MAGIC, PLUGIN_FILTER_VERSION,
	NULL,
	NULL,
	NULL,
	NULL
};

static plugin_descriptor *plugs[]={
	&plugin,0
};

plugin_descriptor **get_plugin_list(void)
{
	return plugs;
}

class PlaylistEditorWatcher: public BHandler
{

	private:
		BList editorlist;
		SoundPlayController *ctrl;

		bool ActivatePlayListEditWindow(PlaylistPtr playlist)
		{
			for(int i=0;;i++)
			{
				Editor *win=(Editor*)editorlist.ItemAt(i);
				if(!win)
					break;
				if(win->Lock())
				{
					if(win->playlist==playlist)
					{
						win->Activate();
						win->Unlock();
						return true;
					}
					win->Unlock();
				}
				else
				{
					editorlist.RemoveItem(win);
					i--;
				}
			}
			return false;
		}

	public:
		PlaylistEditorWatcher(SoundPlayController *controller)
		{
			be_app->AddHandler(this);
			ctrl=controller;
			ctrl->Lock();
			ctrl->AddListener(this);
			ctrl->Unlock();
		}
		~PlaylistEditorWatcher()
		{
			ctrl->Lock();
			ctrl->RemoveListener(this);
			ctrl->Unlock();
			be_app->RemoveHandler(this);
			for(int i=0;;i++)
			{
				Editor *win=(Editor*)editorlist.ItemAt(i);
				if(!win)
					break;
				if(win->Lock())
					win->Quit();
			}
		}
		virtual void MessageReceived(BMessage *msg)
		{
			//msg->PrintToStream();
			switch(msg->what)
			{
				case CONTROLLER_PLAYLISTEDITOR:
				{
					
					int32 id;
					if(B_OK==msg->FindInt32("Playlist",&id))
					{
						PlaylistPtr playlist(id);
						if(playlist->IsValid())
						{
							if(!ActivatePlayListEditWindow(playlist))
								editorlist.AddItem(new Editor(playlist));
						}
					}
				}
				break;
			}
		}

};

// =======================================================================
// plugin struct functions implementation

static void *getplugin(void **data,const char *name, const char *, uint32, plugin_info *info, SoundPlayBufferedReader *reader )
{
	PlaylistEditorWatcher *watcher=new PlaylistEditorWatcher(info->controller);
	
	*data=watcher;
	return &plugin_ops;
}

static void destroyplugin(void *,void *data)
{
	PlaylistEditorWatcher *watcher=(PlaylistEditorWatcher*)data;
	delete watcher;
}


